#include "flash.h"

#include <ch32v00x.h>
#include <ch32v00x_flash.h>

// Disable LOG_DEBUG locally
#undef LOD_DEBUG_ENABLE
#include "log_debug.h"

/* FLASH Keys */
#define RDP_Key                    ((uint16_t)0x00A5)
#define FLASH_KEY1                 ((uint32_t)0x45670123)
#define FLASH_KEY2                 ((uint32_t)0xCDEF89AB)

/* Delay definition */
#define EraseTimeout               ((uint32_t)0x000B0000)
#define ProgramTimeout             ((uint32_t)0x00002000)

/* Flash Control Register bits */
#define CR_PG_Set                  ((uint32_t)0x00000001)
#define CR_PG_Reset                ((uint32_t)0xFFFFFFFE)
#define CR_PER_Set                 ((uint32_t)0x00000002)
#define CR_PER_Reset               ((uint32_t)0xFFFFFFFD)
#define CR_MER_Set                 ((uint32_t)0x00000004)
#define CR_MER_Reset               ((uint32_t)0xFFFFFFFB)
#define CR_OPTPG_Set               ((uint32_t)0x00000010)
#define CR_OPTPG_Reset             ((uint32_t)0xFFFFFFEF)
#define CR_OPTER_Set               ((uint32_t)0x00000020)
#define CR_OPTER_Reset             ((uint32_t)0xFFFFFFDF)
#define CR_STRT_Set                ((uint32_t)0x00000040)
#define CR_LOCK_Set                ((uint32_t)0x00000080)
#define CR_PAGE_PG                 ((uint32_t)0x00010000)
#define CR_PAGE_ER                 ((uint32_t)0x00020000)
#define CR_BUF_LOAD                ((uint32_t)0x00040000)
#define CR_BUF_RST                 ((uint32_t)0x00080000)

static volatile uint16_t *OPTION_BYTES = (uint16_t *)OB_BASE;

/**
 * Default Option bytes values
 * 
 * RDPR  0xA5
 * USER  0x17
 * Data0 0x00
 * Data1 0x00
 * WRPR0 0xFF
 * WRPR1 0xFF
 */

void FLASH_OB_DEBUG() {
    LOG_DEBUG(
        "RDPR 0x%04X\r\n"
        "USER 0x%04X\r\n"
        "Data0 0x%04X\r\n"
        "Data1 0x%04X\r\n"
        "WRPR0 0x%04X\r\n"
        "WRPR1 0x%04X\r\n"
        "\r\n",
        OB->RDPR,
        OB->USER,
        OB->Data0,
        OB->Data1,
        OB->WRPR0,
        OB->WRPR1
    );
}

uint8_t FLASH_OptionByteGet(FLASH_OptionByte_Type byte) {
    uint16_t data = OPTION_BYTES[(uint8_t)byte];
    return (uint8_t)(data & 0xFF);
}

FLASH_Status FLASH_OptionByteSet(FLASH_OptionByte_Type byte, uint8_t data) {
    volatile uint16_t buff[OB_COUNT];
    // Keeo old values
    for (uint8_t i = 0; i < OB_COUNT; i++) {
        buff[i] = OPTION_BYTES[i];
    }
    
    // Set Data0
    buff[byte] = data;

    // Erase
    FLASH_EraseOptionBytes();

    // Write 
    FLASH_Status status = FLASH_COMPLETE;

    FLASH->OBKEYR = FLASH_KEY1;
    FLASH->OBKEYR = FLASH_KEY2;
    status = FLASH_WaitForLastOperation(ProgramTimeout);

    for (uint8_t i = 0; i < OB_COUNT; i++) {
        if (status == FLASH_COMPLETE) {
            FLASH->CTLR |= CR_OPTPG_Set;
            OPTION_BYTES[i] = buff[i];
            status = FLASH_WaitForLastOperation(ProgramTimeout);
            if (status != FLASH_TIMEOUT) {
                FLASH->CTLR &= CR_OPTPG_Reset;
            }
        }
    }

    return status;
}
