/* 
 * Header for using the I2C peripheral in slave mode 
 * for OLIMEX Neo6502PC-PWR board
 * 
 * Project is based on https://github.com/cnlohr/ch32v003fun/blob/master/examples/i2c_slave/
 * 
 * MIT License
 * 
 * Copyright (c) 2024 Renze Nicolai
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __I2C_SLAVE_H
#define __I2C_SLAVE_H

#include <ch32v00x.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

void I2C1_EV_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
void I2C1_EV_IRQHandler(void);

void I2C1_ER_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
void I2C1_ER_IRQHandler(void);


typedef void (*i2c_register_callback_t)(uint8_t reg);
typedef void (*i2c_write_callback_t)(uint8_t reg, uint8_t offset, uint8_t data);
typedef void (*i2c_stop_callback_t)(uint8_t reg, uint8_t offset);
typedef uint8_t (*i2c_read_callback_t)(uint8_t reg, uint8_t offset);

void I2C_Slave_Setup(uint8_t addr);
void I2C_Slave_SetOnRegister(i2c_register_callback_t on_register);
void I2C_Slave_SetOnWrite(i2c_write_callback_t on_write);
void I2C_Slave_SetOnStop(i2c_stop_callback_t on_stop);
void I2C_Slave_SetOnRead(i2c_read_callback_t on_read);

#ifdef __cplusplus
}
#endif

#endif /* __I2C_SLAVE_H */
